%%  Script for generating the figures
%
% Author: D.Fourer (dominique@fourer.fr)
% Date: 19-10-2019
clear
close all

if ~exist('tfrrgab')
 addpath('./hsst')    
end
zpad = 50; %50;                        % zero padding of the signal (can improve the signal reconstruction)

Fs = 1000; %000;
signal = 1; %5;  %% change value in [1,2,3,4]
load_signal

q_method = 2;

%mu_range  = [0.01 0.05 0.2 0.5 1 2 5];      %mu_range = logspace(log10(mu_range(1)), log10(mu_range(2)), NbImg2);
SNR_range = 25; %[45 35 25 15 5];
%w0T_range = [2*pi*2 2*pi*5 2*pi*10];        %w0T_range = linspace(2*pi*1, 2*pi*15, NbImg); % (2*pi*10)          %% range for omega_0 T
w0T_range = 2*pi*2; %2*pi*2; %[1.5 2 3.5];

w0 = 2*pi * 0.2 * Fs;
%L = T/Fs;


%L = 8;
%2*pi*0.1


alpha = 0.3; %25; %0.3;

%Ts = 1/Fs;          %% sampling period
%t  = 0:Ts:(1-Ts);   %% time axis
M  = N+2*zpad; %500;           %% number of frequency bins
M2 = round(M/2);


n  = 1:length(s);   %% time sample
N  = length(n);     %% signal length

s0 = s-mean(s); %% 0-mean signal
s = s0;

%%% prepare folders to store images
chemin0 = 'figs_TSP';

method_name =  {'STFT', 'reassigned spectrogram', ...
                'synchrosqueezed STFT', 'vertical synchrosqueezed STFT', ...
                'time-reassigned synchrosqueezed STFT', 'horizontal synchrosqueezed STFT'};
            
cheminr = sprintf('%s/%d', chemin0, signal);
chemin = { sprintf('%s/stft/',  cheminr), ...
           sprintf('%s/rspec/', cheminr), ...
           sprintf('%s/sstft/', cheminr), ...
           sprintf('%s/vsstft/', cheminr), ...
           sprintf('%s/tsstft/', cheminr),...
           sprintf('%s/thsstft/', cheminr)};
index = ones(1, length(chemin));     

for i = 1:length(chemin)
 if ~exist(chemin{i}, 'dir')
   mkdir(chemin{i});
 end
end

gamma_K = 10^(-4);
f      = ((1:M2)-1)/M * Fs;
nfreqs = f / Fs;

fp = fopen(sprintf('%s/results.txt',cheminr), 'w+b');

for id = 1:length(SNR_range)

 %% add noise
 rsb_target = SNR_range(id);
 s = sigmerge(s0, randn(size(s0)), rsb_target);

 rsb = SNR(s0, s);
  if abs(rsb - rsb_target) > 10
    error('Invalid SNR')
  end
  
 sz       = [zeros(zpad,1);s;zeros(zpad,1)];  %% zero-padded signal
 n_range = (zpad+1):(zpad+length(s0)); 
 
 tmp = sprintf('+++++++++++++  SNR= %0.02f +++++++++++++\n', rsb);
 fprintf(1, tmp); fprintf(fp, tmp);
   
 for idx1 = 1:length(w0T_range)
  w0T = w0T_range(idx1);
  f0T = w0T/(2*pi);
  L = (w0T / w0)*Fs;
  
  tmp = sprintf('++ W0T=%0.02f, f0T=%0.02f, L=%.2f ++\n', w0T, f0T, L);
  fprintf(1, tmp); fprintf(fp, tmp);
 
  %% 1 Compute the classical S-transform and display Stockwellogram
  %[tfr] = tfrst(s, M, w0T, gamma_K);
  [tfr, rtfr] = tfrrgab2(s, M, L, gamma_K);
  %[tfr, rtfr] = tfrrst(s, M, w0T, gamma_K);
  figure(1)
  imagesc(n, nfreqs, (abs(tfr(1:M2,:)).^2).^alpha);
  %set(gca,'Ytick',0:0.05:0.5)
  %set(gca,'Xtick',0:50:N)
  ylim([0 0.499])
  set(gca,'YDir','normal')
  xlabel('time samples', 'FontSize', 16)
  ylabel('normalized frequency', 'FontSize', 16)
  %title(sprintf('Stockwellogram SNR=%0.02f, f_0T=%0.02f', rsb_target, f0T), 'FontSize', 14);
  title(sprintf('SNR=%0.02f dB, L=%0.02f, RE=%0.02f bits', rsb_target, L, renyi_entropy((abs(tfr(1:M2,:)).^2).^alpha,1:N,nfreqs(1:M2))), 'FontSize', 14);
  colormap gray;
  cmap = colormap;
  cmap = flipud(cmap);
  colormap(cmap);
  saveas(gcf, sprintf('%s/stft_%d.eps', chemin{1}, index(1)), 'epsc');
  index(1) = index(1)+1;
 
  %% 1a) Reconstruction classique
  %[s_hat] = rectfrst(tfr);
  [ s_hat ] = real(rectfrgab(tfr, L, M));

  tmp = sprintf('STFT reconstruction, RQF=%0.02f, f_0 T=%0.02f\n', RQF(s,s_hat'), f0T);
  fprintf(1, tmp); fprintf(fp, tmp);


  %% 2 Compute the Reassigned spectrogram
  figure(2)
  imagesc(n, nfreqs, (abs(rtfr(1:M2,:))).^alpha);
  %set(gca,'Ytick',0:0.05:0.5)
  %set(gca,'Xtick',0:50:N)
  ylim([0 0.499])
  set(gca,'YDir','normal')
  xlabel('time samples', 'FontSize', 16)
  ylabel('normalized frequency', 'FontSize', 16)
  %title(sprintf('Reassigned Stockwellogram SNR=%0.02f, f_0T=%0.02f', rsb_target, f0T), 'FontSize', 14);
  title(sprintf('SNR=%0.02f dB, L=%0.02f, RE=%0.02f bits', rsb_target, L, renyi_entropy((abs(rtfr(1:M2,:))).^alpha,1:N,nfreqs(1:M2))), 'FontSize', 14);
  colormap gray;
  cmap = colormap;
  cmap = flipud(cmap);
  colormap(cmap);
  saveas(gcf, sprintf('%s/rstft_%d.eps', chemin{2}, index(2)), 'epsc');
  index(2) = index(2)+1;
%  
%  %% 3 Compute the Synchrosqueezing
%  [~,stfr] = tfrsst(s, M, w0T, gamma_K);
%  
%  figure(3)
%  imagesc(n, nfreqs, (abs(stfr(1:M2,:)).^2).^alpha);
%  %set(gca,'Ytick',0:0.05:0.5)
%  %set(gca,'Xtick',0:50:N)
%  ylim([0 0.499])
%  set(gca,'YDir','normal')
%  xlabel('time samples', 'FontSize', 16)
%  ylabel('normalized frequency', 'FontSize', 16)
%  %title(sprintf('Synchrosqueezed ST SNR=%0.02f, f_0T=%0.02f', rsb_target, f0T), 'FontSize', 14);
%  title(sprintf('SNR=%0.02f, f0T=%0.02f', rsb_target, f0T), 'FontSize', 14);
%  colormap gray;
%  cmap = colormap;
%  cmap = flipud(cmap);
%  colormap(cmap);
%  saveas(gcf, sprintf('%s/sst_%d.eps', chemin{3}, index(3)), 'epsc');
%  index(3) = index(3)+1;
%  
%  %% 3b) Reconstruct the Synchrosqueezed S-transform
%  %s_hat3 = real(rectfrsst(stfr, w0T)); fprintf(1,'Synchrosqueezed ST reconstruction, SNR=%0.02f, f0 T=%0.02f\n', SNR(s,s_hat3'), f0T);
%  
%  %s_hat3 = 2*real(rectfrsst(stfr(1:M2,:), w0T, m_axis(M,1:M2)));
%  s_hat3 = rectfrsst(stfr, w0T);
%  
%  tmp = sprintf('Synchrosqueezed ST reconstruction, SNR=%0.02f, f0 T=%0.02f\n', RQF(s,s_hat3'), f0T);
%  fprintf(1, tmp); fprintf(fp, tmp);

 
 %% x Compute the time-reassigned Synchrosqueezed S-transform
 [~, stfr, lost, lost2] = tfrtsgab2(sz, M, L, gamma_K);
 %[~, stfr, ma, lost,lost2] = tfrtsst(sz, M, w0T, gamma_K);
 figure(5)
 imagesc(n, nfreqs, (abs(stfr(1:M2,n_range)).^2).^alpha);
 %set(gca,'Ytick',0:0.05:0.5)
 %set(gca,'Xtick',0:50:N)
 ylim([0 0.499])
 set(gca,'YDir','normal')
 xlabel('time samples', 'FontSize', 16)
 ylabel('normalized frequency', 'FontSize', 16)
 %title(sprintf('time-reassigned synchrosqueezed ST SNR=%0.02f, f_0T=%0.02f', rsb_target, f0T), 'FontSize', 14);
 title(sprintf('SNR=%0.02f dB, L=%0.02f, RE=%0.02f bits', rsb_target, L, renyi_entropy((abs(stfr(1:M2,n_range)).^2).^alpha,1:N,nfreqs(1:M2))), 'FontSize', 14);
 colormap gray;
 cmap = colormap;
 cmap = flipud(cmap);
 colormap(cmap);
 saveas(gcf, sprintf('%s/tsstft_%d.eps', chemin{5}, index(5)), 'epsc');
 index(5) = index(5)+1;
 
 stfr(:,end) = stfr(:,end) + lost2; %% recover lost energy
 s_hat5 = real(rectfrhsgab(stfr, M));
 s_hat5 = s_hat5(n_range);

 tmp = sprintf('Time-reassigned Synchrosqueezing reconstruction, SNR=%0.02f, f0T=%0.02f\n', RQF(s,s_hat5'), f0T);
 fprintf(1, tmp); fprintf(fp, tmp);

 %% x Compute the second-order time-reassigned Synchrosqueezed S-transform
[~, stfr2, lost, lost2] = tfrthsgab2(sz, M, L,gamma_K, q_method);
 figure(5)
 imagesc(n, nfreqs, (abs(stfr2(1:M2,n_range)).^2).^alpha);
 %set(gca,'Ytick',0:0.05:0.5)
 %set(gca,'Xtick',0:50:N)
 ylim([0 0.499])
 set(gca,'YDir','normal')
 xlabel('time samples', 'FontSize', 16)
 ylabel('normalized frequency', 'FontSize', 16)
 %title(sprintf('horizontal synchrosqueezed ST SNR=%0.02f, f_0T=%0.02f', rsb_target, f0T), 'FontSize', 14);
 title(sprintf('SNR=%0.02f dB, L=%0.02f, RE=%0.02f bits', rsb_target, L, renyi_entropy((abs(stfr2(1:M2,n_range)).^2).^alpha,1:N,nfreqs(1:M2))), 'FontSize', 14);
 colormap gray;
 cmap = colormap;
 cmap = flipud(cmap);
 colormap(cmap);
 saveas(gcf, sprintf('%s/hsstft_%d.eps', chemin{6}, index(6)), 'epsc');
 index(6) = index(6)+1;
 
 stfr2(:,end) = stfr2(:,end) + lost2; %% recover lost energy
 s_hat6 = real(rectfrhsgab(stfr2, M));
 s_hat6 = s_hat6(n_range);
 
 tmp = sprintf('Horizontal Synchrosqueezing reconstruction, SNR=%0.02f, f0T=%0.02f\n', RQF(s,s_hat5'), f0T);
 fprintf(1, tmp); fprintf(fp, tmp);
 
 
 
 
 
 
%   %% 4) Compute the Levenberg-Marquardt Reassigned Stockwellogram
%  for idx2 = 1:length(mu_range)
%   mu = mu_range(idx2);
%   [~, rtfr, lost] = tfrlmrst(s, mu, M, w0T, gamma_K);
%   figure(2)
%   imagesc(n, nfreqs, (abs(rtfr(1:M2,:)).^2).^alpha);
%   %set(gca,'Ytick',0:0.05:0.5)
%   %set(gca,'Xtick',0:50:N)
%   ylim([0 0.499])
%   set(gca,'YDir','normal')
%   xlabel('time samples', 'FontSize', 16)
%   ylabel('normalized frequency', 'FontSize', 16)
%   title(sprintf('LM-Reassigned Stockwellogram SNR=%0.02f, f_0T=%0.02f \\mu=%0.02f', rsb_target, f0T, mu), 'FontSize', 14);
%   %colormap gray;
%   colormap gray;
%   cmap = colormap;
%   cmap = flipud(cmap);
%   colormap(cmap);
%   saveas(gcf, sprintf('%s/rst_%d.eps', chemin{2}, index2), 'epsc');
%   index2 = index2+1;
%  end
%  
% 
%   %% 5) Compute LM synchrosqueezed ST
%  for idx2 = 1:length(mu_range)
%   mu = mu_range(idx2);
%   [~, stfr, lost] = tfrlmsst(s, mu, M, w0T, gamma_K);
%   figure(3)
%   imagesc(n, nfreqs, (abs(stfr(1:M2,:)).^2).^alpha);
%   %set(gca,'Ytick',0:0.05:0.5)
%   %set(gca,'Xtick',0:50:N)
%   ylim([0 0.499])
%   set(gca,'YDir','normal')
%   xlabel('time samples', 'FontSize', 16)
%   ylabel('normalized frequency', 'FontSize', 16)
%   title(sprintf('LM Synchrosqueezed ST SNR=%0.02f dB, f_0T=%0.02f \\mu=%0.02f', rsb_target, f0T, mu), 'FontSize', 14);
%   colormap gray;
%   cmap = colormap;
%   cmap = flipud(cmap);
%   colormap(cmap);
%   saveas(gcf, sprintf('%s/sst_%d.eps', chemin{3}, index3), 'epsc');
%   index3 = index3+1;
%   
%   s_hat3 = rectfrsst(stfr, w0T);
%   tmp = sprintf('Synchrosqueezed ST reconstruction, RQF=%0.02f dB, mu=%0.02f, f0T=%0.02f\n', RQF(s,s_hat3'), mu, f0T);
%   fprintf(1, tmp); fprintf(fp, tmp);
%  end
%  
%  %% 6) Compute the Vertical second-order synchsqueezed ST
%  [~,stfr] = tfrvsst(s, M, w0T, gamma_K);
%  figure(4)
%  imagesc(n, nfreqs, (abs(stfr(1:M2,:)).^2).^alpha);
%  %set(gca,'Ytick',0:0.05:0.5)
%  %set(gca,'Xtick',0:50:N)
%  ylim([0 0.499])
%  set(gca,'YDir','normal')
%  xlabel('time samples', 'FontSize', 16)
%  ylabel('normalized frequency', 'FontSize', 16)
%  title(sprintf('Vertical Synchrosqueezed ST SNR=%0.02f, f_0T=%0.02f', rsb_target, f0T), 'FontSize', 14);
%  colormap gray;
%  cmap = colormap;
%  cmap = flipud(cmap);
%  colormap(cmap);
%  saveas(gcf, sprintf('%s/vsst_%d.eps', chemin{4}, index4), 'epsc');
%  index4 = index4+1;
 
 end    %% w0T
end  %% SNR

fclose all;
for i = 1:length(chemin)
 eps2pdf(chemin{i});
end